/*
 * @plotXYHistogram.java Version 1.1 07/20/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.awt.event.*;

import plot.plotXYStruct;
import plot.plotSymbolStruct;
import plot.plotSymbol;

/** Class plotXY
 *  <p> This class will create a generic X-Y Plot for an array of data
 *
 *  @version 1.1 07/20/2011
 *  @author  John R. Victorine
 */

public class plotXYHistogram extends Canvas
{
  private int              iPlot   = -1;
  private plotXYStruct     stPlot  = null;
  private String           sPLOT    = "";

  // Plot Dimensions

  private int              iXBegin = 80;    // Left Edge of Plot Area
  private int              iYBegin = 60;    // Top Edge of Plot Area
  private int              iYSep   = 60;    // Top Edge of Plot Area
  private int              iWidth  = 500;   // Width of Plot Area
  private int              iHeight = 200;   // Height of Plot Area
  private int              iSep    = 250;
//  private int              iHeight = 500; //300;    // Height of Plot Area

  // Plot Data

  private int              iRows = 0;  // Total Number of Rows
  private int              iClmn = 0;  // Total Number of Columns
  private int              iX[][];     // Array of X Data converted to Pixels
  private int              iY[][];     // Array of Y Data converted to Pixels
  private int              iZ[][];     // Array of X Data converted to Pixels
  private plotSymbolStruct stSymbol[]; // Array of Symbol and Color for Point

  private int              iCells = 100; // Total number of cells
  // Legend

  private int              iLegend = 0; // Total Number of Legends
  private plotSymbolStruct stLegend[];  // Array of Symbol and Color for Legend

  // Plot Symbols

  private plotSymbol pSymbol = new plotSymbol();

  // Filter Extra Data Indicator

  private boolean bFlag = false;

  /** Constructor plotXYHistogram()
   *  <p> This is the Constructor for this class.
   *  @param iPlot  = the Plot Type
   *  @param stPlot = Plot Data Structure.
   *  @param sPLOT    = Type of Plot, "2D" or "3D"
   */

  public plotXYHistogram(int iPlot, plotXYStruct stPlot, String sPLOT)
  {
    setBackground(Color.white);

    this.iPlot  = iPlot;
    this.stPlot = stPlot;
    this.sPLOT  = new String( sPLOT );

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    iRows = stPlot.iRows;

    iCells = 50; //100;
//    if (iRows < 1000) { iCells = (10 * iRows) / 100; }
//    if (iCells < 10)  { iCells = 10; }

    if (stPlot.stSymbols != null)
      this.stSymbol = stPlot.stSymbols;

    this.iLegend = stPlot.iLegend;
    if (stPlot.stLegend != null)
      this.stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setData(plotXYStruct stPlot)
   * <p> This method will change the Plot XY Data Structure
   * @param stPlot = the Plot XY Data Structure
   */

  public void setData(plotXYStruct stPlot)
  {
	int i=0;

    this.stPlot = stPlot;

    if (stPlot.iWidth != 0)
      iWidth = stPlot.iWidth;

    if (stPlot.iHeight != 0)
      iHeight = stPlot.iHeight;

    iRows = stPlot.iRows;

    iCells = 50; //100;
//    if (iRows < 1000) { iCells = (10 * iRows) / 100; }
//    if (iCells < 10)  { iCells = 10; }

    stSymbol = null;
    if (stPlot.stSymbols != null)
      stSymbol = stPlot.stSymbols;

    stLegend = null;
    iLegend  = stPlot.iLegend;
    if (stPlot.stLegend != null)
      stLegend = stPlot.stLegend;

    // Convert Raw Data to Pixels

    convertDataToPixels();
    repaint();
  }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    this.iLegend  = iLegend;
    this.stLegend = stLegend;
  }

  /** Method setLegendFlag()
   * <p> This method will set the Legend on or off by toggling the iLegend
   *     value
   * @param iLegend = The total number of legend values
   */

  public void setLegendFlag(int iLegend)
  {
    this.iLegend = iLegend;
    repaint();
  }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
    this.stPlot.sTitle1 = new String(sText);
    repaint();
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
    this.stPlot.sTitle2 = new String(sText);
    repaint();
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
    this.stPlot.sTitle3 = new String(sText);
    repaint();
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sXAxis      = new String(sText);
    this.stPlot.iXLog       = iGrid;
    this.stPlot.dXMaximum   = dMax;
    this.stPlot.dXMinimum   = dMin;
    this.stPlot.dXIncrement = dInc;
    this.stPlot.iXCycles    = iCycle;

    convertDataToPixels();
    repaint();
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sYAxis      = new String(sText);
    this.stPlot.iYLog       = iGrid;
    this.stPlot.dYMaximum   = dMax;
    this.stPlot.dYMinimum   = dMin;
    this.stPlot.dYIncrement = dInc;
    this.stPlot.iYCycles    = iCycle;

    convertDataToPixels();
    repaint();
  }

  /** Method setZAxis()
   * <p> This method will reset the Z-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setZAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    this.stPlot.sZAxis      = new String(sText);
    this.stPlot.iZLog       = iGrid;
    this.stPlot.dZMaximum   = dMax;
    this.stPlot.dZMinimum   = dMin;
    this.stPlot.dZIncrement = dInc;
    this.stPlot.iZCycles    = iCycle;

    convertDataToPixels();
    repaint();
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag)
  {
    this.bFlag = bFlag;
    repaint();
  }

  /** Method setTriangle()
   * <p> This method will set the triangle data
   * @param iData      = The total number points
   * @param sLithology = The lithology values
   * @param dMat       = Rhomaa-Umaa values
   */

  public void setTriangle(int iData, String sLithology[], double dMat[][])
  {
  }

  /* =============================================================== *
   * ----------------------- DELETE METHODS ------------------------ *
   * =============================================================== */

  /** Method close()
   * <p> This method will set strings and arrays to null so Java can
   *     automatically clear memory
   */

  public void close()
  {
    stPlot   = null;

    iX       = null; // Array of X Data converted to Pixels
    iY       = null; // Array of Y Data converted to Pixels
    iZ       = null; // Array of Z Data converted to Pixels
    stSymbol = null; // Array of Symbol and Color for Point
    stLegend = null; // Array of Symbol and Color for Legend

    sPLOT    = null;

    if (pSymbol != null)
      pSymbol.close();
    pSymbol  = null;
  }

  /* ================================================================== *
   * ------------------------ SUPPORT METHODS ------------------------- *
   * ================================================================== */

  /** Method void convertDataToPixels()
   *  <p> This method will set the Plot Data
   */

  private void convertDataToPixels()
  {
    int    i,j,k;
    int    iRatio = 0;
    double dXMin  = 0;
    double dXMax  = 0;
    double dYMin  = 0;
    double dYMax  = 0;
    double dZMin  = 0;
    double dZMax  = 0;
    double del    = 0.0;
    double dRatio = 0.0;
    int    XMax   = 0;
    int    YMax   = 0;
    int    ZMax   = 0;
    int    ittlx  = 0;
    int    ittly  = 0;
    int    ittlz  = 0;
    int    icont  = 0;

    iRows = stPlot.iRows;

    if (stPlot.bLegend)
    {
	  if (stPlot.iLegend > 0)
	  {
		iClmn = iLegend;
      }
    }
    else
    {
	  iClmn = 1;
	}

    iX = new int[iCells][iClmn];
    iY = new int[iCells][iClmn];
    iZ = new int[iCells][iClmn];

    for (j=0; j<iCells-1; j++)
	{
      for (k=0; k<iClmn; k++)
	  {
	    iX[j][k] = 0;
	    iY[j][k] = 0;
	    iZ[j][k] = 0;
	  }
	}

    // Set the X-Axis Data

    // IF the X-Axis Data is LINEAR THEN
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
    {
	  del = (stPlot.dXMaximum - stPlot.dXMinimum)/iCells;
      for (i=0; i<iRows; i++)
      {
		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((stPlot.dX[i] >= stPlot.dXMinimum+j*del) &&
		        (stPlot.dX[i] < stPlot.dXMinimum+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
			    }
			  }

			  if (icont == 1) { iX[j][k]++; }
		    }
	      }
		}
      }
    }
    // IF the X-Axis Data is LOG THEN
    else
    {
      dXMin = Math.log(stPlot.dXMinimum) / Math.log(10.0);
      dXMax = Math.log(stPlot.dXMaximum) / Math.log(10.0);
	  del   = (dXMax - dXMin)/iCells;

      for (i=0; i<iRows; i++)
      {
		dRatio = Math.log(stPlot.dX[i])/Math.log(10.0);

		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((dRatio >= dXMin+j*del) &&
		        (dRatio < dXMin+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
  			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
				}
			  }

		      if (icont == 1) { iX[j][k]++; }
	        }
	      }
		}
      }
    }

    // Set the Y-Axis Data

    // IF the Y-Axis Data is LINEAR THEN
    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
    {
	  del = (stPlot.dYMaximum - stPlot.dYMinimum)/iCells;
      for (i=0; i<iRows; i++)
      {
		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((stPlot.dY[i] >= stPlot.dYMinimum+j*del) &&
		        (stPlot.dY[i] < stPlot.dYMinimum+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
				}
			  }

			  if (icont == 1) { iY[j][k]++; }
	        }
	      }
		}
      }
    }
    // IF the Y-Axis Data is LOG THEN
    else
    {
      dYMin = Math.log(stPlot.dYMinimum) / Math.log(10.0);
      dYMax = Math.log(stPlot.dYMaximum) / Math.log(10.0);
	  del   = (dYMax - dYMin)/iCells;

      for (i=0; i<iRows; i++)
      {
		dRatio = Math.log(stPlot.dY[i])/Math.log(10.0);

		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((dRatio >= dYMin+j*del) &&
		        (dRatio < dYMin+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
				}
			  }

			  if (icont == 1) { iY[j][k]++; }
	        }
	      }
		}
      }
    }

    // Set the Z-Axis Data

    // IF the Z-Axis Data is LINEAR THEN
    if (stPlot.iZLog == plot.plotXYStruct.LINEAR)
    {
	  del = (stPlot.dZMaximum - stPlot.dZMinimum)/iCells;
      for (i=0; i<iRows; i++)
      {
		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((stPlot.dZ[i] >= stPlot.dZMinimum+j*del) &&
		        (stPlot.dZ[i] < stPlot.dZMinimum+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
			    }
			  }

			  if (icont == 1) { iZ[j][k]++; }
		    }
	      }
		}
      }
    }
    // IF the Z-Axis Data is LOG THEN
    else
    {
      dZMin = Math.log(stPlot.dZMinimum) / Math.log(10.0);
      dZMax = Math.log(stPlot.dZMaximum) / Math.log(10.0);
	  del   = (dZMax - dZMin)/iCells;

      for (i=0; i<iRows; i++)
      {
		dRatio = Math.log(stPlot.dZ[i])/Math.log(10.0);

		for (j=0; j<iCells-1; j++)
		{
          for (k=0; k<iClmn; k++)
     	  {
		    if ((dRatio >= dZMin+j*del) &&
		        (dRatio < dZMin+(j+1)*del))
		    {
			  icont = 1;
			  if (iClmn > 1)
			  {
				if (stLegend != null)
				{
  			      if (!stPlot.sZ[i].equals(stLegend[k].symbol))
			        icont = 0;
				}
			  }

		      if (icont == 1) { iZ[j][k]++; }
	        }
	      }
		}
      }
    }

    for (j=0; j<iCells-1; j++)
	{
	  ittlx = 0;
	  ittly = 0;
	  ittlz = 0;
      for (k=0; k<iClmn; k++)
  	  {
		ittlx = ittlx + iX[j][k];
		ittly = ittly + iY[j][k];
		ittlz = ittlz + iZ[j][k];
      }

      if (ittlx > XMax) { XMax = ittlx; }
      if (ittly > YMax) { YMax = ittly; }
      if (ittlz > ZMax) { ZMax = ittlz; }
	}

    for (j=0; j<iCells-1; j++)
	{
      for (k=0; k<iClmn; k++)
  	  {
        if (XMax > 0) { iX[j][k] = iHeight * iX[j][k] / XMax; }
	    if (YMax > 0) { iY[j][k] = iHeight * iY[j][k] / YMax; }
	    if (ZMax > 0) { iZ[j][k] = iHeight * iZ[j][k] / ZMax; }
      }
    }
  }

  /* ================================================================== *
   * -------------------------- DRAW METHODS -------------------------- *
   * ================================================================== */

  /** Method drawTitles()
   *  <p> This method will draw the titles of the Plot.
   *  @param g - Graphics pointer.
   */

  public void drawTitles(Graphics g)
  {
    int iTemp = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    if (stPlot.sTitle1 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle1.length())));
      g.drawString(stPlot.sTitle1, iTemp / 2, 10);
    }

    if (stPlot.sTitle2 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle2.length())));
      g.drawString(stPlot.sTitle2, iTemp / 2, 30);
    }

    if (stPlot.sTitle3 != null)
    {
      iTemp = (int) (Math.abs(iWidth - (6 * stPlot.sTitle3.length())));
      g.drawString(stPlot.sTitle3, iTemp / 2, 50);
    }
  }

  /** Method drawLabels()
   *  <p> This method will draw the Linear Plot Labels.
   *  @param g      = Graphics pointer.
   *  @param iY     = starting height of plot track
   *  @param sLabel = Axis Label
   */

  public void drawLabels(Graphics g, int iY, String sLabel)
  {
    int ivalue;

    g.setColor(Color.black);

    if (sLabel != null)
    {
      ivalue = (iWidth+100)/2 - 3 * (sLabel).length();
      g.drawString(sLabel, ivalue, iY+iYBegin+iHeight+35);
    }
  }

  /** Method drawGrid()
   *  <p> This method will draw the Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    if (stPlot.iXLog == plot.plotXYStruct.LINEAR)
      drawLinearXAxisGrid(g,    0, stPlot.dXMinimum, stPlot.dXMaximum, stPlot.dXIncrement);
    else
      drawLogXAxisGrid(g,    0, stPlot.dXMinimum, stPlot.dXMaximum, stPlot.dXIncrement);

    if (stPlot.iYLog == plot.plotXYStruct.LINEAR)
      drawLinearXAxisGrid(g, iSep, stPlot.dYMinimum, stPlot.dYMaximum, stPlot.dYIncrement);
    else
      drawLogXAxisGrid(g, iSep, stPlot.dYMinimum, stPlot.dYMaximum, stPlot.dYIncrement);

    if (sPLOT.equals("3D"))
    {
      if (stPlot.iZLog == plot.plotXYStruct.LINEAR)
        drawLinearXAxisGrid(g, 2*iSep, stPlot.dZMinimum, stPlot.dZMaximum, stPlot.dZIncrement);
      else
        drawLogXAxisGrid(g, 2*iSep, stPlot.dZMinimum, stPlot.dZMaximum, stPlot.dZIncrement);
    }
  }

  /** Method drawLinearXAxisGrid()
   *  <p> This method will draw the Linear X-Axis Grid Lines
   *  @param g  = Graphics pointer.
   *  @param iY = starting height of plot track
   */

  public void drawLinearXAxisGrid(
	  Graphics g, int iY, double dMin, double dMax, double dIncr)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    iXinc = (int) ((double)iWidth * dIncr /
                   (dMax - dMin));

    idraw = iXBegin;
    while (idraw < iXBegin+iWidth+5)
    {
      dx     = dMin + (double) i * dIncr;
      iRatio = (int) ((double) iWidth * (dx - dMin) / (dMax - dMin));
      idraw = iXBegin + iRatio;

      if (idraw < iXBegin+iWidth+5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(idraw, iY+iYBegin+iHeight-6, idraw, iY+iYBegin+iHeight);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );
        dx = Math.round(dx*1000)/1000.0;
        g.drawString(""+dx, idraw, iY+iYBegin+iHeight+15);

        for (j=1; j<5; j++)
        {
          idel = idraw + (j * iXinc) / 5;
          if (idel < iXBegin+iWidth)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idel, iY+iYBegin+iHeight-6, idel, iY+iYBegin+iHeight);
          }
        }
      }

      i++;
    }
  }

  /** Method drawLogXAxisGrid()
   *  <p> This method will draw the Log X-Axis Grid Lines
   *  @param g  = Graphics pointer.
   *  @param iY = starting height of plot track
   */

  public void drawLogXAxisGrid(
	  Graphics g, int iY, double dMin, double dMax, double dIncr)
  {
    int    i, j;
    int    l=1;
    int    k=0;
    int    iXinc;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc;
    double dTemp;
    double dXinc;

    iXinc = 1 + iWidth / stPlot.iXCycles;
    if (iY > iSep)
      iXinc = 1 + iWidth / stPlot.iZCycles;
    else if (iY > 0)
      iXinc = 1 + iWidth / stPlot.iYCycles;

    g.setColor(Color.darkGray);
    g.drawLine(iXBegin, iY+iYBegin+iHeight-6, iXBegin, iY+iYBegin+iHeight);

    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);
      g.drawLine(i, iY+iYBegin+iHeight-6, i, iY+iYBegin+iHeight);

      g.setColor(Color.lightGray);
      for (j=2; j<10; j++)
      {
        dXinc = (double) j;
        dTemp = 1.0+(double) iXinc * Math.log(dXinc) / Math.log(10.0);
        jXinc = i - iXinc + (int) dTemp;

        g.drawLine(jXinc, iY+iYBegin+iHeight-6, jXinc, iY+iYBegin+iHeight);
      }
    }

    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    k=1;
    del = dIncr;
    g.drawString("" + dMin, iXBegin-10, iY+iYBegin+iHeight+15);
    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
      g.setColor(Color.darkGray);

      dLabel = (double) k * 10.0 * dMin;
      k = k * 10;

      g.setColor(Color.black);
      g.drawString("" + dLabel, i-10, iY+iYBegin+iHeight+15);
      del = dIncr + del;
    }
  }

  /** Method drawXPlotData()
   * <p> This method will X plot the data
   *  @param g  = Graphics pointer.
   *  @param iYo = starting height of plot track
   */

  public void drawXPlotData(Graphics g, int iYo)
  {
	int idel  = 0;
    int iClrU = 255;
    int iClr  = 0;
    int ittl  = 0;

    if (iCells > 0)
      idel  = iWidth / iCells;
    else
      idel  = iWidth;

    for (int i=0; i<iCells; i++)
    {
      ittl  = 0;
      for (int k=0; k<iClmn; k++)
  	  {
        iClr = iClrU - iX[i][k];
        g.setColor(new Color( 0, iClr, 0 ));

        if (stPlot.bLegend)
	    {
		  if (stLegend != null)
		  {
            g.setColor(new Color(stLegend[k].iRed,
                                 stLegend[k].iGreen,
                                 stLegend[k].iBlue));
	 	  }
   	    }

  	    g.fillRect(iXBegin+i*idel, iYo+iYBegin+iHeight-(ittl+iX[i][k]),
  	               idel, iX[i][k]);
	    ittl = ittl + iX[i][k];
      }
	}
  }

  /** Method drawYPlotData()
   * <p> This method will X plot the data
   *  @param g  = Graphics pointer.
   *  @param iYo = starting height of plot track
   */

  public void drawYPlotData(Graphics g, int iYo)
  {
	int idel  = 0;
    int iClrU = 255;
    int iClr  = 0;
    int ittl  = 0;

    if (iCells > 0)
      idel  = iWidth / iCells;
    else
      idel  = iWidth;

    for (int i=0; i<iCells; i++)
    {
	  ittl  = 0;
      for (int k=0; k<iClmn; k++)
  	  {
        iClr = iClrU - iY[i][k];
	    g.setColor(new Color( iClr, 0, 0 ));

        if (stPlot.bLegend)
	    {
		  if (stLegend != null)
		  {
            g.setColor(new Color(stLegend[k].iRed,
                                 stLegend[k].iGreen,
                                 stLegend[k].iBlue));
		  }
   	    }

  	    g.fillRect(iXBegin+i*idel, iYo+iYBegin+iHeight-(ittl+iY[i][k]),
  	               idel, iY[i][k]);
	    ittl = ittl + iY[i][k];
      }
	}
  }

  /** Method drawZPlotData()
   * <p> This method will Z plot the data
   *  @param g  = Graphics pointer.
   *  @param iYo = starting height of plot track
   */

  public void drawZPlotData(Graphics g, int iYo)
  {
	int idel  = 0;
    int iClrU = 255;
    int iClr  = 0;
    int ittl  = 0;

    if (iCells > 0)
      idel  = iWidth / iCells;
    else
      idel  = iWidth;

    for (int i=0; i<iCells; i++)
    {
      ittl  = 0;
      for (int k=0; k<iClmn; k++)
  	  {
        iClr = iClrU - iZ[i][k];
        g.setColor(new Color( 0, 0, iClr ));

        if (stPlot.bLegend)
	    {
		  if (stLegend != null)
		  {
            g.setColor(new Color(stLegend[k].iRed,
                                 stLegend[k].iGreen,
                                 stLegend[k].iBlue));
	 	  }
   	    }

  	    g.fillRect(iXBegin+i*idel, iYo+iYBegin+iHeight-(ittl+iZ[i][k]),
  	               idel, iZ[i][k]);
	    ittl = ittl + iZ[i][k];
      }
	}
  }

  /** Method drawLegend()
   *  <p> This method will create the Legend
   *  @param g        = Graphics pointer.
   */

  public void drawLegend(Graphics g)
  {
    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    g.setColor(Color.black);
    if (stPlot.sLegTitle.length() > 0)
      g.drawString(stPlot.sLegTitle, iXBegin+iWidth+10+20, iYBegin-13);

    for (int i=0; i<iLegend; i++)
    {
      g.setColor(new Color(stLegend[i].iRed,
                           stLegend[i].iGreen,
                           stLegend[i].iBlue));
      g.fillRect(iXBegin+iWidth+10+20, iYBegin-8+i*13, 8, 8);

      g.setColor(Color.black);
      g.drawString(stLegend[i].symbol, iXBegin+iWidth+10+32, iYBegin+i*13);
    }
  }

  /** Method draw()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void draw(Graphics g)
  {
    drawTitles(g);

    drawLabels(g, 0,   stPlot.sXAxis);
    drawLabels(g, iSep, stPlot.sYAxis);
    if (sPLOT.equals("3D"))
    {
      drawLabels(g, 2*iSep, stPlot.sZAxis);
    }

    drawXPlotData(g, 0);
    drawYPlotData(g, iSep);
    if (sPLOT.equals("3D"))
    {
      drawZPlotData(g, 2*iSep);
    }

    drawGrid(g);
    g.setColor(Color.black);
    if (stPlot.sLegRange.length() > 0)
      g.drawString(stPlot.sLegRange, iXBegin+iWidth+10+20, iYBegin-26);
    if (stPlot.bLegend) drawLegend(g);
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	draw(g);
  }
}