/*
 * @quakeAnimationLimitsFrame.java Version 1.1 05/25/2011
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import quake.quakePlotFilterListStruct; // Plot Filter List Structure
import plot.plotSymbolStruct;           // Symbol Data Structure
import plot.plotXYStruct;               // XY Data Structure

/** Class quakeAnimationLimitsFrame
 *  <p> This class will display the Cross Plot Control Frame
 *
 *  @version 1.1 05/25/2011
 *  @author  John R. Victorine
 */

public class quakeAnimationLimitsFrame extends JFrame implements ActionListener
{
  public final static int _MAGNITUDE = quake.quakePlotConstants._MAGNITUDE;
  public final static int _DATE      = quake.quakePlotConstants._DATE;
  public final static int _DEPTH     = quake.quakePlotConstants._DEPTH;

  private static final int _OFF      = plot.plotXYZData._OFF;
  private static final int _ON       = plot.plotXYZData._ON;

  // Input Variables

  private int                     iAction  = _ON;
  private int                     iTRS     = _ON;
  private Observable              notifier = null; // Observalbles Object
  private plotXYStruct            stPlotXY = null; // XY Plot Data Structure

  // Global Variables

  private int                     iPanel   = _MAGNITUDE;

  // -- Plot Control Data

  private String                  sTitle1 = "Kansas Geological Survey";
  private String                  sTitle2 = "Kansas Earthquakes";

  private String                  sXAxis  = "Longitude";
  private int                     iXLog   = 0;
  private double                  dXMin   = -98.35;
  private double                  dXMax   = -97.1; //-97.35;
  private double                  dXIncr  =   0.25;

  private String                  sYAxis  = "Latitude";
  private int                     iYLog   = 0;
  private double                  dYMin   = 37.0;
  private double                  dYMax   = 37.5;
  private double                  dYIncr  = 0.1;

  private String                  sZAxis  = "Depth (km)";
  private int                     iZLog   = 0;
  private double                  dZMin   = -15.0;
  private double                  dZMax   =   0.0;
  private double                  dZIncr  =   1.0;

  // -- Filter Data By

  private quakePlotFilterListStruct stLegend =
            quake.quakePlotConstants.getFilter(); // Legend Filter List Data Structure

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField   txtTitle1  = new JTextField();
  private JTextField   txtTitle2  = new JTextField();
//  private JTextField   txtTitle3  = new JTextField();

  // .. X-Axis Widgets

  private JTextField   txtXAxis   = new JTextField();
  private JTextField   txtXMin    = new JTextField();
  private JTextField   txtXMax    = new JTextField();
  private JTextField   txtXIncr   = new JTextField();

  // .. Y-Axis Widgets

  private JTextField   txtYAxis   = new JTextField();
  private JTextField   txtYMin    = new JTextField();
  private JTextField   txtYMax    = new JTextField();
  private JTextField   txtYIncr   = new JTextField();

  // .. Z-Axis Widgets

  private JTextField   txtZAxis   = new JTextField();
  private JTextField   txtZMin    = new JTextField();
  private JTextField   txtZMax    = new JTextField();
  private JTextField   txtZIncr   = new JTextField();

  // .. Wells on or off

  private JRadioButton rbOFF      = new JRadioButton();
  private JRadioButton rbON       = new JRadioButton();

  // .. TRS on or off

  private JRadioButton rbTRSOFF = new JRadioButton();
  private JRadioButton rbTRSON  = new JRadioButton();

  // .. Build Default Plot Structure Button

  private JButton      btnBuild   = new JButton();

  // -- Menus
  // .. File Button Menu

  private JMenuItem mExit = null; // Exit Plot

  /** Constructor quakeAnimationLimitsFrame()
   *  <p> This is the constructor for this class
   * @param stData   = Earthquake Data Structure
   */

  public quakeAnimationLimitsFrame( Observable   notifier,
                                    int          iAction,
                                    int          iTRS,
                                    plotXYStruct stPlotXY )
  {
    try
    {
      this.notifier = notifier;
      this.iAction  = iAction;
      this.iTRS     = iTRS;
      this.stPlotXY = stPlotXY;

      setPlotStruct();

      jbInit();

      if (iAction == _ON)
	    rbON.setSelected(true);
	  else
	    rbOFF.setSelected(true);

      if (iTRS == _ON)
	    rbTRSON.setSelected(true);
	  else
	    rbTRSOFF.setSelected(true);

      addWindowListener( new quakeAnimationLimitsFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel   pnlTop      = new JPanel();
    JPanel   pnlTitles   = new JPanel();
    JPanel   pnlAxis     = new JPanel();
    JPanel   pnlXAxis    = new JPanel();
    JPanel   pnlYAxis    = new JPanel();
    JPanel   pnlZAxis    = new JPanel();
    JPanel   pnlBase     = new JPanel();
    JPanel   pnlBase1    = new JPanel();
    JPanel   pnlFilter   = new JPanel();
    JPanel   pnlStatus   = new JPanel();
    JPanel   pnlTRS      = new JPanel();

    ButtonGroup groupS   = new ButtonGroup();
    ButtonGroup groupTRS = new ButtonGroup();

    TitledBorder titledBorderFilter = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Filter Data By:");
    titledBorderFilter.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStatus = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Toggle Wells:");
    titledBorderStatus.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTRS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "TRS Grid:");
    titledBorderTRS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    JMenuBar menuBar     = new JMenuBar();
    JMenu    menuFile    = new JMenu("File");

    // Build Menus

    // -- Menu Bar

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    menuFile.add( mExit );

    menuBar.add(menuFile);
    this.setJMenuBar(menuBar);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( "Kansas Earthquake 3D Plot Control" );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Titles Panel

    pnlTitles = buildPlotTitlesPanel();

    // -- Axes Panel

    pnlAxis.setLayout(new GridLayout(3,1));

    pnlXAxis = buildPlotXAxisPanel();
    pnlYAxis = buildPlotYAxisPanel();
    pnlZAxis = buildPlotZAxisPanel();

    // Buttons Panel

//    pnlBase.setLayout(new GridLayout(2,1));
    pnlBase.setLayout(new BorderLayout());

    // -- Filter By Buttons

    pnlFilter.setLayout(new BorderLayout());

    btnBuild.setText("Apply Changes");
    btnBuild.addActionListener(this);

    // Turn Well Status On or Off

    pnlStatus.setLayout(new GridLayout());
    pnlStatus.setBorder(titledBorderStatus);

    rbON.setFont(new java.awt.Font("Dialog", 0, 11));
    rbON.setHorizontalAlignment(AbstractButton.CENTER);
    rbON.setSelected(true);
    rbON.setText("On");
    rbON.addActionListener(this);

    rbOFF.setFont(new java.awt.Font("Dialog", 0, 11));
    rbOFF.setHorizontalAlignment(AbstractButton.CENTER);
    rbOFF.setText("Off");
    rbOFF.addActionListener(this);

    // TRS Grid On or Off

    pnlBase1.setLayout(new BorderLayout());

    pnlTRS.setLayout(new GridLayout());
    pnlTRS.setBorder(titledBorderTRS);

    rbTRSON.setFont(new java.awt.Font("Dialog", 0, 11));
    rbTRSON.setHorizontalAlignment(AbstractButton.CENTER);
    rbTRSON.setSelected(true);
    rbTRSON.setText("On");
    rbTRSON.addActionListener(this);

    rbTRSOFF.setFont(new java.awt.Font("Dialog", 0, 11));
    rbTRSOFF.setHorizontalAlignment(AbstractButton.CENTER);
    rbTRSOFF.setText("Off");
    rbTRSOFF.addActionListener(this);

    // Attach Widgets

    this.getContentPane().add(pnlTop, BorderLayout.NORTH);
    pnlTop.add(pnlTitles,             BorderLayout.NORTH);

    pnlTop.add(pnlAxis,               BorderLayout.CENTER);
    pnlAxis.add(pnlXAxis,             null);
    pnlAxis.add(pnlYAxis,             null);
    pnlAxis.add(pnlZAxis,             null);

    pnlTop.add(pnlBase,               BorderLayout.SOUTH);
    pnlBase.add(pnlStatus,            BorderLayout.NORTH);
    pnlStatus.add(rbON,               null);
    pnlStatus.add(rbOFF,              null);

    groupS.add(rbON);
    groupS.add(rbOFF);

    pnlBase.add(pnlBase1,             BorderLayout.CENTER);
    pnlBase1.add(pnlTRS,              BorderLayout.NORTH);
    pnlTRS.add(rbTRSON,               null);
    pnlTRS.add(rbTRSOFF,              null);

    groupTRS.add(rbTRSON);
    groupTRS.add(rbTRSOFF);

    pnlBase.add(pnlFilter,            BorderLayout.SOUTH);
    pnlFilter.add(btnBuild,           BorderLayout.CENTER);

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(350, 550));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation( (d.width - this.getSize().width)/2,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method buildPlotTitlesPanel()
   *  <p> This method will build the Plot Titles Panel
   */

  private JPanel buildPlotTitlesPanel()
  {
	JPanel panel     = new JPanel();
    JPanel pnlTitle1 = new JPanel();
    JPanel pnlTitle2 = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Title:");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Title:");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Titles Panel

    panel.setLayout(new GridLayout(2,1));

    // .. First Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText("");
    txtTitle1.setText(sTitle1);
    txtTitle1.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle1.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. Second Title Panel

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText("");
    txtTitle2.setText(sTitle2);
    txtTitle2.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle2.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

	// Attach Widgets

    panel.add(pnlTitle1,     null);
    pnlTitle1.add(txtTitle1, BorderLayout.CENTER);

    panel.add(pnlTitle2,     null);
    pnlTitle2.add(txtTitle2, BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotXAxisPanel()
   *  <p> This method will build the Plot XAxis Panel
   */

  private JPanel buildPlotXAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlXAxis      = new JPanel();
    JPanel pnlXAxisRange = new JPanel();
    JPanel pnlXMin       = new JPanel();
    JPanel pnlXMax       = new JPanel();
    JPanel pnlXIncr      = new JPanel();

    TitledBorder titledBorderXAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

    txtXAxis.setText("");
    txtXAxis.setText(sXAxis);
    txtXAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtXAxis.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. X-Axis Range Panel

    pnlXAxisRange.setLayout(new GridLayout());

    // .. .. X-Axis Minimum Panel

    pnlXMin.setLayout(new BorderLayout());
    pnlXMin.setBorder(titledBorderMin);

    txtXMin.setText("");
    txtXMin.setText(""+dXMin);
    txtXMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMin.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. X-Axis Maximum Panel

    pnlXMax.setLayout(new BorderLayout());
    pnlXMax.setBorder(titledBorderMax);

    txtXMax.setText("");
    txtXMax.setText(""+dXMax);
    txtXMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMax.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. X-Axis Increment Panel

    pnlXIncr.setLayout(new BorderLayout());
    pnlXIncr.setBorder(titledBorderIncr);

    txtXIncr.setText("");
    txtXIncr.setText(""+dXIncr);
    txtXIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXIncr.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    panel.add(pnlXAxis,         BorderLayout.CENTER);
    pnlXAxis.add(txtXAxis,      BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisRange, BorderLayout.SOUTH);
    pnlXAxisRange.add(pnlXMin,  null);
    pnlXMin.add(txtXMin,        BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXMax,  null);
    pnlXMax.add(txtXMax,        BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXIncr, null);
    pnlXIncr.add(txtXIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotYAxisPanel()
   *  <p> This method will build the Plot YAxis Panel
   */

  private JPanel buildPlotYAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlYAxis      = new JPanel();
    JPanel pnlYAxisRange = new JPanel();
    JPanel pnlYMin       = new JPanel();
    JPanel pnlYMax       = new JPanel();
    JPanel pnlYIncr      = new JPanel();

    TitledBorder titledBorderYAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText("");
    txtYAxis.setText(sYAxis);
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
    pnlYMin.setBorder(titledBorderMin);

    txtYMin.setText("");
    txtYMin.setText(""+dYMin);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
    pnlYMax.setBorder(titledBorderMax);

    txtYMax.setText("");
    txtYMax.setText(""+dYMax);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
    pnlYIncr.setBorder(titledBorderIncr);

    txtYIncr.setText("");
    txtYIncr.setText(""+dYIncr);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    panel.add(pnlYAxis,         BorderLayout.CENTER);
    pnlYAxis.add(txtYAxis,      BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange, BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,  null);
    pnlYMin.add(txtYMin,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,  null);
    pnlYMax.add(txtYMax,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr, null);
    pnlYIncr.add(txtYIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotZAxisPanel()
   *  <p> This method will build the Plot ZAxis Panel
   */

  private JPanel buildPlotZAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlZAxis      = new JPanel();
    JPanel pnlZAxisRange = new JPanel();
    JPanel pnlZMin       = new JPanel();
    JPanel pnlZMax       = new JPanel();
    JPanel pnlZIncr      = new JPanel();

    TitledBorder titledBorderZAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Z-Axis:");
    titledBorderZAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // Z-Axis Panel

    pnlZAxis.setLayout(new BorderLayout());
    pnlZAxis.setBorder(titledBorderZAxis);

    txtZAxis.setText("");
    txtZAxis.setText(sZAxis);
    txtZAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtZAxis.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. Z-Axis Range Panel

    pnlZAxisRange.setLayout(new GridLayout());

    // .. .. Z-Axis Minimum Panel

    pnlZMin.setLayout(new BorderLayout());
    pnlZMin.setBorder(titledBorderMin);

    txtZMin.setText("");
    txtZMin.setText(""+dZMin);
    txtZMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZMin.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. Z-Axis Maximum Panel

    pnlZMax.setLayout(new BorderLayout());
    pnlZMax.setBorder(titledBorderMax);

    txtZMax.setText("");
    txtZMax.setText(""+dZMax);
    txtZMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZMax.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    // .. .. Z-Axis Increment Panel

    pnlZIncr.setLayout(new BorderLayout());
    pnlZIncr.setBorder(titledBorderIncr);

    txtZIncr.setText("");
    txtZIncr.setText(""+dZIncr);
    txtZIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZIncr.addFocusListener(new quakeAnimationLimitsFrameFocusAdapter(this));

    panel.add(pnlZAxis,         BorderLayout.CENTER);
    pnlZAxis.add(txtZAxis,      BorderLayout.NORTH);

    pnlZAxis.add(pnlZAxisRange, BorderLayout.SOUTH);
    pnlZAxisRange.add(pnlZMin,  null);
    pnlZMin.add(txtZMin,        BorderLayout.CENTER);

    pnlZAxisRange.add(pnlZMax,  null);
    pnlZMax.add(txtZMax,        BorderLayout.CENTER);

    pnlZAxisRange.add(pnlZIncr, null);
    pnlZIncr.add(txtZIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD quakeAnimationLimitsFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quakeAnimationLimitsFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      if (notifier != null)
        notifier.notifyObservers( new String("Close Limits Frame") );
//	  close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    notifier = null;

    sTitle1 = null;
    sTitle2 = null;

    sXAxis  = null;
    sYAxis  = null;
    sZAxis  = null;

    stPlotXY   = null;

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    // Global Frame Widgets
    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;

    // .. X-Axis Widgets

    txtXAxis  = null;
    txtXMin   = null;
    txtXMax   = null;
    txtXIncr  = null;

    // .. Y-Axis Widgets

    txtYAxis  = null;
    txtYMin   = null;
    txtYMax   = null;
    txtYIncr  = null;

    // .. Z-Axis Widgets

    txtZAxis  = null;
    txtZMin   = null;
    txtZMax   = null;
    txtZIncr  = null;

    // .. Filter By Buttons

    rbON      = null;
    rbOFF     = null;

    rbTRSON   = null;
    rbTRSOFF  = null;

    btnBuild  = null;

    // -- Menus

    mExit     = null; // Exit

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getWellStatus()
   *  <p> This method will return the Well Status
   */

  public int getWellStatus() { return (iAction); }

  /** Method getGridStatus()
   *  <p> This method will return the Well Status
   */

  public int getGridStatus() { return (iTRS); }

  /** Method getPlotStruct()
   *  <p> This method will return the XY Plot Data Structure
   */

  public plotXYStruct getPlotStruct()
  {
	stPlotXY             = new plotXYStruct();

    stPlotXY.sTitle1     = new String( sTitle1 );
    stPlotXY.sTitle2     = new String( sTitle2 );
    stPlotXY.sTitle3     = new String( "" );

    stPlotXY.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotXY.iXCycles    = 1;      // Number of Cycles
    stPlotXY.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotXY.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotXY.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotXY.sXMnemonic  = new String( "" );
    stPlotXY.sXAxis      = new String( sXAxis );

    stPlotXY.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotXY.iYCycles    = 1;      // Number of Cycles
    stPlotXY.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotXY.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotXY.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotXY.sYMnemonic  = new String( "" );
    stPlotXY.sYAxis      = new String( sYAxis );

    stPlotXY.iZLog       = 0;      // Z-Axis Semilog Indicator
    stPlotXY.iZCycles    = 1;      // Number of Cycles
    stPlotXY.dZMaximum   = dZMax;  // Maximum Z-Axis
    stPlotXY.dZMinimum   = dZMin;  // Minimum Z-Axis
    stPlotXY.dZIncrement = dZIncr; // Increment along the Z-Axis
    stPlotXY.sZMnemonic  = new String( "" );
    stPlotXY.sZAxis      = new String( sZAxis );

    stLegend = quake.quakePlotConstants.getFilter();

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Magnitude:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (int i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

    return (stPlotXY);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setPlotStruct()
   * <p> This method will set the Plot Structure
   */

  private void setPlotStruct()
  {
    sTitle1 = new String( stPlotXY.sTitle1 );
    sTitle2 = new String( stPlotXY.sTitle2 );

    dXMax   = stPlotXY.dXMaximum;   // Maximum X-Axis
    dXMin   = stPlotXY.dXMinimum;   // Minimum X-Axis
    dXIncr  = stPlotXY.dXIncrement; // Increment along the X-Axis
    sXAxis  = new String( stPlotXY.sXAxis );

    dYMax   = stPlotXY.dYMaximum;   // Maximum Y-Axis
    dYMin   = stPlotXY.dYMinimum;   // Minimum Y-Axis
    dYIncr  = stPlotXY.dYIncrement; // Increment along the Y-Axis
    sYAxis  = new String( stPlotXY.sYAxis );

    dZMax   = stPlotXY.dZMaximum;   // Maximum Z-Axis
    dZMin   = stPlotXY.dZMinimum;   // Minimum Z-Axis
    dZIncr  = stPlotXY.dZIncrement; // Increment along the Z-Axis
    sZAxis  = new String( stPlotXY.sZAxis );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbON ) { iAction = _ON;  }
    if (event.getSource() == rbOFF) { iAction = _OFF; }

    if (event.getSource() == rbTRSON ) { iTRS = _ON;  }
    if (event.getSource() == rbTRSOFF) { iTRS = _OFF; }

    if (event.getSource() == btnBuild)
    {
      if (notifier != null)
        notifier.notifyObservers( new String("Apply Changes") );
	}

    // Exit Panel

	if (event.getSource() == mExit)
	{
      if (notifier != null)
        notifier.notifyObservers( new String("Close Limits Frame") );
//	  close();
    }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if (e.getSource() == txtTitle1)
    {
      sTitle1 = new String( txtTitle1.getText() );
	}

    if (e.getSource() == txtTitle2)
    {
      sTitle2 = new String( txtTitle2.getText() );
	}

    // .. X-Axis Widgets

    if (e.getSource() == txtXAxis)
    {
      sXAxis = new String( txtXAxis.getText() );
	}
    if (e.getSource() == txtXMin )
    {
      iNumeric = 1;
      sTemp = txtXMin.getText();
      sMessage = new String("X-Minimum Value is a Numeric Field");

      s1 = txtXMin.getText();
      s2 = txtXMax.getText();

      if (s1.equals(s2))
      {
		txtXMin.setText(""+stPlotXY.dXMinimum);
//		txtXMax.setText(""+stPlotXY.dXMaximum);

		sTemp = new String("A");
        sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtXMin.setText(""+stPlotXY.dXMinimum);

		  sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtXMax )
    {
      iNumeric = 1;
      sTemp = txtXMax.getText();
      sMessage = new String("X-Maximum Value is a Numeric Field");

      s1 = txtXMin.getText();
      s2 = txtXMax.getText();

      if (s1.equals(s2))
      {
//		txtXMin.setText(""+stPlotXY.dXMinimum);
		txtXMax.setText(""+stPlotXY.dXMaximum);

		sTemp = new String("A");
        sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtXMax.setText(""+stPlotXY.dXMaximum);

		  sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtXIncr)
    {
      iNumeric = 1;
      sTemp = txtXIncr.getText();
      sMessage = new String("X-Increment Value is a Numeric Field");

      s3 = txtXIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtXIncr.setText(""+stPlotXY.dXIncrement);
		if (stPlotXY.iXLog == 1) { txtYIncr.setText(""+stPlotXY.iXCycles); }
		sTemp = new String("A");
        sMessage = new String("X-Increment Value can not be 0");
	  }
	}

    // .. Y-Axis Widgets

    if (e.getSource() == txtYAxis)
    {
      stPlotXY.sYAxis = new String( txtYAxis.getText() );
	}

    if (e.getSource() == txtYMin )
    {
      iNumeric = 1;
      sTemp = txtYMin.getText();
      sMessage = new String("Y-Minimum Value is a Numeric Field");

      s1 = txtYMin.getText();
      s2 = txtYMax.getText();

      if (s1.equals(s2))
      {
		txtYMin.setText(""+stPlotXY.dYMinimum);
//		txtYMax.setText(""+stPlotXY.dYMaximum);

		sTemp = new String("A");
        sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtYMin.setText(""+stPlotXY.dYMinimum);

		  sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtYMax )
    {
      iNumeric = 1;
      sTemp = txtYMax.getText();
      sMessage = new String("Y-Maximum Value is a Numeric Field");

      s1 = txtYMin.getText();
      s2 = txtYMax.getText();

      if (s1.equals(s2))
      {
//		txtYMin.setText(""+stPlotXY.dYMinimum);
		txtYMax.setText(""+stPlotXY.dYMaximum);

		sTemp = new String("A");
        sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtYMax.setText(""+stPlotXY.dYMaximum);

		  sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtYIncr)
    {
      iNumeric = 1;
      sTemp = txtYIncr.getText();
      sMessage = new String("Y-Increment Value is a Numeric Field");

      s3 = txtYIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtYIncr.setText(""+stPlotXY.dYIncrement);
		if (stPlotXY.iYLog == 1) { txtYIncr.setText(""+stPlotXY.iYCycles); }
		sTemp = new String("A");
        sMessage = new String("Y-Increment Value can not be 0");
	  }
	}

    // .. Z-Axis Widgets

    if (e.getSource() == txtZAxis)
    {
      stPlotXY.sZAxis = new String( txtZAxis.getText() );
	}

    if (e.getSource() == txtZMin )
    {
      iNumeric = 1;
      sTemp = txtZMin.getText();
      sMessage = new String("Z-Minimum Value is a Numeric Field");

      s1 = txtZMin.getText();
      s2 = txtZMax.getText();

      if (s1.equals(s2))
      {
		txtZMin.setText(""+stPlotXY.dZMinimum);
		txtZMax.setText(""+stPlotXY.dZMaximum);

		sTemp = new String("A");
        sMessage = new String("Z-Minimum Value can not equal Z-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtZMin.setText(""+stPlotXY.dZMinimum);

		  sTemp = new String("A");
          sMessage = new String("Z-Minimum Value can not be greater than Z-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtZMax )
    {
      iNumeric = 1;
      sTemp = txtZMax.getText();
      sMessage = new String("Z-Maximum Value is a Numeric Field");

      s1 = txtZMin.getText();
      s2 = txtZMax.getText();

      if (s1.equals(s2))
      {
		txtZMin.setText(""+stPlotXY.dZMinimum);
		txtZMax.setText(""+stPlotXY.dZMaximum);

		sTemp = new String("A");
        sMessage = new String("Z-Minimum Value can not equal Z-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtZMax.setText(""+stPlotXY.dZMaximum);

		  sTemp = new String("A");
          sMessage = new String("Z-Minimum Value can not be greater than Z-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtZIncr)
    {
      iNumeric = 1;
      sTemp = txtZIncr.getText();
      sMessage = new String("Z-Increment Value is a Numeric Field");

      s3 = txtZIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtZIncr.setText(""+stPlotXY.dYIncrement);
		if (stPlotXY.iZLog == 1) { txtZIncr.setText(""+stPlotXY.iZCycles); }
		sTemp = new String("A");
        sMessage = new String("Z-Increment Value can not be 0");
	  }
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
      }
      else
      {
        if ((e.getSource() == txtXMin ) ||
            (e.getSource() == txtXMax ) ||
            (e.getSource() == txtXIncr ))
        {
          if (e.getSource() == txtXMin )
            dXMin = cmn.cmnString.stringToDouble(txtXMin.getText());

          if (e.getSource() == txtXMax )
            dXMax = cmn.cmnString.stringToDouble(txtXMax.getText());

          if (e.getSource() == txtXIncr )
          {
			dXIncr = cmn.cmnString.stringToDouble(txtXIncr.getText());
		  }
	    }

        if ((e.getSource() == txtYMin ) ||
            (e.getSource() == txtYMax ) ||
            (e.getSource() == txtYIncr ))
        {
          if (e.getSource() == txtYMin )
            dYMin = cmn.cmnString.stringToDouble(txtYMin.getText());

          if (e.getSource() == txtYMax )
            dYMax = cmn.cmnString.stringToDouble(txtYMax.getText());

          if (e.getSource() == txtYIncr )
			dYIncr = cmn.cmnString.stringToDouble(txtYIncr.getText());
		}

        if ((e.getSource() == txtZMin ) ||
            (e.getSource() == txtZMax ) ||
            (e.getSource() == txtZIncr ))
        {
          if (e.getSource() == txtZMin )
            dZMin = cmn.cmnString.stringToDouble(txtZMin.getText());

          if (e.getSource() == txtZMax )
            dZMax = cmn.cmnString.stringToDouble(txtZMax.getText());

          if (e.getSource() == txtZIncr )
			dZIncr = cmn.cmnString.stringToDouble(txtZIncr.getText());
	    }
      }
    }
  }
}

/** CLASS quakeAnimationLimitsFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class quakeAnimationLimitsFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  quakeAnimationLimitsFrame adaptee;

  quakeAnimationLimitsFrameFocusAdapter(quakeAnimationLimitsFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 05/25/2011
 *  @author  John Victorine
 */
