/*
 * @quakeAnimateUtility.java Version 1.1 08/27/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import java.sql.Timestamp;

import quake.quakeSeisStruct;  // Seismic Data Structure

import plot.plotXYListStruct;  // Plot XY List Data Structure
import plot.plotXYStruct;      // Plot XY Data Structure
import plot.plotSymbolStruct;  // Symbol Data Structure

/** Class quakeAnimateUtility
 *  <p> This is the quake seismic data structure utility class.
 *
 *  @version 1.1 08/27/2015
 *  @author  John R. Victorine
 */

public class quakeAnimateUtility
{
  public static final String sTitle1 = "Kansas Geological Survey";
  public static final String sTitle2 = "Kansas Earthquakes - Animation";
  public static final String sTitle3 = "";

  public static final String sXAxis  = "Longitude";
  public static final int    iXLog   = 0;
  public static final double dXMin   = -98.35;
  public static final double dXMax   = -97.1; //-97.35;
  public static final double dXIncr  =   0.25;

  public static final String sYAxis  = "Latitude";
  public static final int    iYLog   = 0;
  public static final double dYMin   = 37.0;
  public static final double dYMax   = 37.5;
  public static final double dYIncr  = 0.1;

  public static final String sZAxis  = "Depth (km)";
  public static final int    iZLog   = 0;
  public static final double dZMin   = -15.0;
  public static final double dZMax   =   0.0;
  public static final double dZIncr  =   1.0;

  public static final quakePlotFilterListStruct stLegend =
                        quake.quakePlotConstants.getFilter();

  /** Method getData()
   *  <p> This method will build the Plot XY List Data Structure
   *      to create the effect of animation of the Earthquake data.
   *  @param  st  = Seismic Data Structure
   *  @return stP = Plot XY List Data Structure
   */

  public static plotXYListStruct getData(quakeSeisStruct st, plotXYStruct stPo)
  {
	int i=0;
	plotXYListStruct stP   = null;
	plotXYStruct     stTmp = null;
	int              icnt  = 0;
	int              ittl  = 0;
    double           data  = 0.0;

	if (st != null)
	{
	  if (st.iRows > 0)
	  {
		for (i=0; i<st.iRows; i++)
		{
          if (isDepth( st.depth[i], st.mag[i], stPo ))
          {
            ittl++;
          }
		}

        stP        = new plotXYListStruct();
		stP.stItem = new plotXYStruct[ittl];

		for (i=0; i<st.iRows; i++)
		{
          if (isDepth( st.depth[i], st.mag[i], stPo ))
          {
			stTmp = copy(stPo);
			stTmp.sTitle3 = new String( st.ts[i].toString() );
			stTmp = addData( i, st, stTmp, st.ts[i] );

//System.out.println( stTmp.dZ[0]+" "+stTmp.dY[0]+" "+stTmp.dX[0]+" "+stTmp.sZ[0]);

			stP   = plot.plotXYUtility.add(stP, stTmp);

			icnt++;

			stTmp.delete();
			stTmp = null;
		  }
		}

		stP.iCount = icnt;
	  }
	}

	return (stP);
  }

  /** Method defaultPlotStruct()
   * <p> This method will set default Plot Structure
   *
   * @return stPlotXY = Plot XY Data Structure
   */

  public static plotXYStruct defaultPlotStruct()
  {
	plotXYStruct stPlotXY = new plotXYStruct();

    stPlotXY.sTitle1     = new String( sTitle1 );
    stPlotXY.sTitle2     = new String( sTitle2 );
    stPlotXY.sTitle3     = new String( sTitle3 );

    stPlotXY.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotXY.iXCycles    = 1;      // Number of Cycles
    stPlotXY.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotXY.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotXY.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotXY.sXMnemonic  = new String( "" );
    stPlotXY.sXAxis      = new String( sXAxis );

    stPlotXY.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotXY.iYCycles    = 1;      // Number of Cycles
    stPlotXY.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotXY.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotXY.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotXY.sYMnemonic  = new String( "" );
    stPlotXY.sYAxis      = new String( sYAxis );

    stPlotXY.iZLog       = 0;      // Z-Axis Semilog Indicator
    stPlotXY.iZCycles    = 1;      // Number of Cycles
    stPlotXY.dZMaximum   = dZMax;  // Maximum Z-Axis
    stPlotXY.dZMinimum   = dZMin;  // Minimum Z-Axis
    stPlotXY.dZIncrement = dZIncr; // Increment along the Z-Axis
    stPlotXY.sZMnemonic  = new String( "" );
    stPlotXY.sZAxis      = new String( sZAxis );

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Magnitude:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (int i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

    return (stPlotXY);
  }

  /** Method copy()
   * <p> This method will copy one data structure to another
   * @param  stOld = The Old Data Structure
   * @return stNew = The New Data Structure
   */

  public static plotXYStruct copy(plotXYStruct stOld)
  {
    plotXYStruct stNew = null;

    if (stOld != null)
    {
	  stNew             = new plotXYStruct();

      stNew.iWidth      = stOld.iWidth;      // Width of the Plot Area
      stNew.iHeight     = stOld.iHeight;     // Height of the Plot Area

      stNew.iWo         = stOld.iWo;         // Width of the 2D Plot Area
      stNew.iHo         = stOld.iHo;         // Height of the 2D Plot Area

      // Titles

      stNew.sTitle1     = new String(stOld.sTitle1); // The First Line of the Title
      stNew.sTitle2     = new String(stOld.sTitle2); // The Second Line of the Title
      stNew.sTitle3     = new String(stOld.sTitle3); // The Third Line of the Title

      // Step Size

      stNew.dStep       = stOld.dStep;

      // X-Axis Values.

      stNew.iXLog       = stOld.iXLog;       // X-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iXCycles    = stOld.iXCycles;    // Number of Cycles
      stNew.dXMaximum   = stOld.dXMaximum;   // Maximum X-Axis
      stNew.dXMinimum   = stOld.dXMinimum;   // Minimum X-Axis
      stNew.dXIncrement = stOld.dXIncrement; // Increment along the X-Axis
      stNew.sXMnemonic  = new String(stOld.sXMnemonic); // X-Axis Mnemonic
      stNew.sXAxis      = new String(stOld.sXAxis);     // X-Axis Label.

      // Y-Axis Values.

      stNew.iYLog       = stOld.iYLog;       // Y-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iYCycles    = stOld.iYCycles;    // Number of Cycles
      stNew.dYMaximum   = stOld.dYMaximum;   // Maximum Y-Axis
      stNew.dYMinimum   = stOld.dYMinimum;   // Minimum Y-Axis
      stNew.dYIncrement = stOld.dYIncrement; // Increment along the Y-Axis
      stNew.sYMnemonic  = new String(stOld.sYMnemonic); // Y-Axis Mnemonic
      stNew.sYAxis      = new String(stOld.sYAxis);     // Y-Axis Label.

      // Z-Axis Values.

      stNew.iZLog       = stOld.iZLog;       // Z-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iZCycles    = stOld.iZCycles;    // Number of Cycles
      stNew.dZMaximum   = stOld.dZMaximum;   // Maximum Z-Axis
      stNew.dZMinimum   = stOld.dZMinimum;   // Minimum Z-Axis
      stNew.dZIncrement = stOld.dZIncrement; // Increment along the Z-Axis
      stNew.sZMnemonic  = new String(stOld.sZMnemonic); // Z-Axis Mnemonic
      stNew.sZAxis      = new String(stOld.sZAxis);     // Z-Axis Label.

      // Legend

      stNew.sLegRange  = new String(stOld.sLegRange); // Depth Range
      stNew.sLegTitle  = new String(stOld.sLegTitle); // Legend Title
      stNew.bLegend    = stOld.bLegend;               // Legend OFF
      stNew.iLegend    = stOld.iLegend;               // Number of Legends
      stNew.stLegend   = new plotSymbolStruct[stNew.iLegend]; // Array of Symbols & colors

      for (int i=0; i<stNew.iLegend; i++)
      {
        stNew.stLegend[i] = plot.plotSymbol.getPlotSymbolStructure(
                              stOld.stLegend[i].iEmpty,
                              stOld.stLegend[i].iSymbol,
                              stOld.stLegend[i].iRed,
                              stOld.stLegend[i].iGreen,
                              stOld.stLegend[i].iBlue,
                              stOld.stLegend[i].symbol );
	  }
    }

    return (stNew);
  }

  /** Method addData()
   * <p> This method will add the plot data
   * @param  iRow     = the row number in raw data to plot
   * @param  stData   = the seismic data structure
   * @param  stPlotXY = the Plot XY Data Structure
   * @return stPlotXY = the Plot XY Data Structure
   */

  public static plotXYStruct addData( int             iRow,
                                      quakeSeisStruct stData,
                                      plotXYStruct    stPlotXY )
  {
    int    i        = 0;
    int    iRed     = 0;
    int    iGreen   = 0;
    int    iBlue    = 0;
    int    iFilter  = -1;
    int    iArray   = -1;

    stPlotXY.iRows  = 1;             // Number of Rows
    stPlotXY.depths = new double[1]; // Array of Depths
    stPlotXY.dX     = new double[1]; // Array of Horizontal Axis Data
    stPlotXY.dY     = new double[1]; // Array of Vertical Axis Data
    stPlotXY.dZ     = new double[1]; // Array of Other Data
    stPlotXY.sZ     = new String[1]; // Array of Description
    stPlotXY.stSymbols = new plotSymbolStruct[1]; // Array of Symbol


	stPlotXY.depths[0] = stData.depth[iRow];
    stPlotXY.dZ[0]     = stData.depth[iRow];
    stPlotXY.dY[0]     = stData.dLatitude[iRow];
    stPlotXY.dX[0]     = stData.dLongitude[iRow];
    stPlotXY.sZ[0]     = new String("");

    if (stData.src[iRow].equals("CO2 Sensor Array"))
      iArray = 1;
    else
      iArray = -1;

    iFilter = getFilter( stData.mag[iRow] );
    if (iFilter == 999)
    {
      stPlotXY.stSymbols[0] =
        plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.OVAL,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
    }
    else
    {
   	  stPlotXY.sZ[0] = new String( stLegend.stItem[iFilter].symbol );

	  if (iArray == -1)
	  {
  	    stPlotXY.stSymbols[0] =
		  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[iFilter].iSymbol,
                    stLegend.stItem[iFilter].iRed,
                    stLegend.stItem[iFilter].iGreen,
                    stLegend.stItem[iFilter].iBlue,
                    stLegend.stItem[iFilter].symbol);
	  }
	  else
	  {
		stPlotXY.stSymbols[0] =
	      plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.TRIANGLE,
                    stLegend.stItem[iFilter].iRed,
                    stLegend.stItem[iFilter].iGreen,
                    stLegend.stItem[iFilter].iBlue,
                    stLegend.stItem[iFilter].symbol);
	  }
	}

	return ( stPlotXY );
  }

  /** Method addData()
   * <p> This method will add the plot data
   * @param  iRow     = the row number in raw data to plot
   * @param  stData   = the seismic data structure
   * @param  stPlotXY = the Plot XY Data Structure
   * @param  ts       = Data & Time of event
   * @return stPlotXY = the Plot XY Data Structure
   */

  public static plotXYStruct addData( int             iRow,
	                                  quakeSeisStruct stData,
                                      plotXYStruct    stPlotXY,
                                      Timestamp       ts )
  {
    int    i        = 0;
    int    iRed     = 0;
    int    iGreen   = 0;
    int    iBlue    = 0;
    int    iFilter  = -1;
    int    iArray   = -1;
    int    iCount   = 0;
    int    iDays    = 0;
    int    iDepth   = -1;

    // Clear the Plot Data Structure

    if (stPlotXY != null)
    {
      stPlotXY.dX = null; // Array of Horizontal Axis Data
      stPlotXY.dY = null; // Array of Vertical Axis Data
      stPlotXY.dZ = null; // Array of Other Data
      stPlotXY.sZ = null; // Array of Description

      if (stPlotXY.stSymbols != null)
      {
	    for(i=0; i<stPlotXY.iRows; i++)
	    {
		  if (stPlotXY.stSymbols[i] != null)
		  {
		    stPlotXY.stSymbols[i].delete();
	    	stPlotXY.stSymbols[i] = null;
	      }
	    }
        stPlotXY.stSymbols = null;
      }
	}

    if (stData != null)
    {
      for (i=0; i<stData.iRows; i++)
      {
        if (isDate(stData.ts[iRow], stData.ts[i]))
        {
		  iCount++;
		}
	  }

      stPlotXY.iRows  = iCount;             // Number of Rows
      stPlotXY.depths = new double[iCount]; // Array of Depths
      stPlotXY.dX     = new double[iCount]; // Array of Horizontal Axis Data
      stPlotXY.dY     = new double[iCount]; // Array of Vertical Axis Data
      stPlotXY.dZ     = new double[iCount]; // Array of Other Data
      stPlotXY.sZ     = new String[iCount]; // Array of Description
      stPlotXY.stSymbols = new plotSymbolStruct[iCount]; // Array of Symbol

      iCount = 0;

      for (i=0; i<stData.iRows; i++)
      {
        if (isDate(stData.ts[iRow], stData.ts[i]))
        {
          iDepth = isGreaterThan5km(stData.depth[i], stData.dLatitude[i], stData.dLongitude[i]);

	      if (iCount < stPlotXY.iRows)
	      {
			stPlotXY.depths[iCount] = stData.depth[i];
            stPlotXY.dZ[iCount]     = stData.depth[i];
            stPlotXY.dY[iCount]     = stData.dLatitude[i];
            stPlotXY.dX[iCount]     = stData.dLongitude[i];
            stPlotXY.sZ[iCount]     = new String("");

	        iDays = getDays(stData.ts[iRow], stData.ts[i]);

            if (stData.src[i].equals("CO2 Sensor Array"))
              iArray = 1;
            else
              iArray = -1;

            iFilter = getFilter( stData.mag[i] );
            if (iFilter == 999)
            {
              stPlotXY.stSymbols[iCount] =
                plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.OVAL,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
            }
            else
            {
	          switch (iDays)
	          {
			    case 0:
                  iRed   = stLegend.stItem[iFilter].iRed;
                  iGreen = stLegend.stItem[iFilter].iGreen;
                  iBlue  = stLegend.stItem[iFilter].iBlue;
			      break;

			    case 1:
			      iRed   = 127;
			      iGreen = 127;
			      iBlue  = 127;
			      break;

			    case 2:
			      iRed   = 150;
			      iGreen = 150;
			      iBlue  = 150;
			      break;

			    case 3:
			      iRed   = 190;
			      iGreen = 190;
			      iBlue  = 190;
			      break;

			    case 4:
			      iRed   = 229;
			      iGreen = 229;
			      iBlue  = 229;
			      break;

			    case 5:
			      iRed   = 248;
			      iGreen = 248;
			      iBlue  = 255;
			      break;
			  }

   			  stPlotXY.sZ[iCount] = new String(
			    stLegend.stItem[iFilter].symbol );

			  if (iArray == -1)
			  {
				stPlotXY.stSymbols[iCount] =
		          plot.plotSymbol.getPlotSymbolStructure(
                      plot.plotSymbolStruct.FILLED,
                      stLegend.stItem[iFilter].iSymbol,
                      iRed,
                      iGreen,
                      iBlue,
                      stLegend.stItem[iFilter].symbol);
			  }
			  else
			  {
				if (iDepth == -1)
				{
				  stPlotXY.stSymbols[iCount] =
		            plot.plotSymbol.getPlotSymbolStructure(
                        plot.plotSymbolStruct.FILLED,
                        plot.plotSymbolStruct.TRIANGLE,
                        iRed,
                        iGreen,
                        iBlue,
                        stLegend.stItem[iFilter].symbol);
			    }
			    else
			    {
				  stPlotXY.stSymbols[iCount] =
		            plot.plotSymbol.getPlotSymbolStructure(
                        plot.plotSymbolStruct.EMPTY,
                        plot.plotSymbolStruct.TRIANGLE,
                        iRed,
                        iGreen,
                        iBlue,
                        stLegend.stItem[iFilter].symbol);
				}
			  }
		    }

            iCount++;
		  }
		}
	  }
	}

	return ( stPlotXY );
  }

  /** Method isGreaterThan5km()
   *  <p> This method will determine if the distance between Wellington KGS 2-32
   *      and seismic is greater than or equal to 5 km
   *
   *        Wellington KGS 2-32
   *           Longitude: -97.4418
   *           Latitude:   37.3105345
   *
   *       Finding distances based on Latitude and Longitude
   *           { http://andrew.hedges.name/experiments/haversine/ }
   *           "This JavaScript uses the Haversine Formula (shown below) expressed
   *            in terms of a two-argument inverse tangent function to calculate the
   *            great circle distance between two points on the Earth. This is the
   *            method recommended for calculating short distances by Bob Chamberlain
   *            (rgc@jpl.nasa.gov) of Caltech and NASA's Jet Propulsion Laboratory as
   *            described on the U.S. Census Bureau Web site."
   *
   *       dlon = lon2 - lon1
   *       dlat = lat2 - lat1
   *       a = (sin(dlat/2))^2 + cos(lat1) * cos(lat2) * (sin(dlon/2))^2
   *       c = 2 * atan2( sqrt(a), sqrt(1-a) )
   *       d = R * c (where R is the radius of the Earth)
   *
   *  @param  depth = depth of earthquake
   *  @param  dLat  = Latitude location of earthquake
   *  @param  dLong = Longitude location of earthquake
   *  @return iKM   = indicator that the distance is greater than or equal to 5 km (1) or not (-1)
   */

  private static int isGreaterThan5km(double depth, double dLat, double dLong)
  {
	int iKM = -1;
	double d232Lat  = 37.3105345;
	double d232Long = -97.4418;
	double R        = 6373.0; // Radius of the Earth (km)
	double delLat   = 0.0;
    double delLong  = 0.0;
    double a        = 0.0;
    double c        = 0.0;
    double d        = 0.0;
    double depth1   = Math.abs(depth);

    delLong = Math.PI * (dLong - d232Long) / 180.0;
    delLat  = Math.PI * (dLat  - d232Lat) / 180.0;

    a = Math.sin(delLat/2) * Math.sin(delLat/2) +
        Math.cos(d232Lat) * Math.cos(dLat) * Math.sin(delLong/2) * Math.sin(delLong/2);

    c = 2.0 * Math.atan2( Math.sqrt(a), Math.sqrt(1.0-a) );
    d = Math.sqrt( (R*c)*(R*c) + depth1*depth1 );

    if (d >= 5)
      iKM = 1;

	return (iKM);
  }

  /** Method isDepth()
   *  <p> This method will check if the data is part of the filter
   *  @param  depth    = the first depth
   *  @param  data     = the data value
   *  @param  stPlotXY = Plot XY Data Structure
   *  @return bValue   = true, if the data within filter, false otherwise
   */

  public static boolean isDepth( double depth, double data, plotXYStruct stPlotXY  )
  {
	boolean bValue = false;

    if ((depth >= stPlotXY.dZMinimum) && (depth <= stPlotXY.dZMaximum))
    {
      if (stLegend != null)
      {
        for (int i=0; i<stLegend.iCount; i++)
        {
  	      if ((data >= stLegend.stItem[i].dataStart) &&
		      (data < stLegend.stItem[i].dataEnd))
		  {
  	        bValue = true;
		  }
		}
	  }
	}

	return (bValue);
  }

  /** Method getFilter()
   *  <p> This method will check if the data is part of the filter
   *  @param  data   = the data value
   *  @return iValue = Legend
   */

  public static int getFilter( double data )
  {
	int iValue = -1;

    if (stLegend != null)
    {
      for (int i=0; i<stLegend.iCount; i++)
      {
        if ((data >= stLegend.stItem[i].dataStart) &&
	        (data < stLegend.stItem[i].dataEnd))
		{
  	      iValue = i;
		}
      }
	}

	return (iValue);
  }

  /** Method isDate()
   *  <p> This method will only allow so many minutes
   *  @param ts   = date needed
   *  @param ts2  = date of event
   */

  public static boolean isDate(Timestamp ts, Timestamp ts2)
  {
	boolean bValue = false;
	long    lday5  = (long) (5 * 1440 * 60 * 1000);

	long lnow      = ts.getTime();
	long lmin      = ts.getTime() - lday5;
	long ldata     = ts2.getTime();

	if ((ldata <= lnow) && (ldata > lmin))
	  bValue = true;

	return (bValue);
  }

  /** Method getDays()
   *  <p> This method will only allow so many minutes
   *  @param ts   = date needed
   *  @param ts2  = date of event
   */

  public static int getDays(Timestamp ts, Timestamp ts2)
  {
	int  days = 0;
	long day1 = (long) (1440 * 60 * 1000);

	long lnow  = ts.getTime();
	long ldata = ts2.getTime();
    long ldiff = lnow - ldata;

    days = (int) ( ldiff / day1 );

	return (days);
  }
}

/*
 *  @version 1.1 08/27/2015
 *  @author  John Victorine
 */
