/*
 * @quakePlotFilterUtility.java Version 1.1 08/17/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import quake.quakePlotFilterListStruct;
import quake.quakePlotFilterStruct;

/** Class quakePlotFilterUtility
 *  <p> This class will hold the filter ranges Utility
 *
 *  @version 1.1 08/17/2015
 *  @author  John R. Victorine
 */

public class quakePlotFilterUtility
{
  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static quakePlotFilterListStruct copyList( quakePlotFilterListStruct stOld )
  {
    quakePlotFilterListStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new quakePlotFilterListStruct();
      stNew.iCount     = stOld.iCount;
      stNew.stItem     = new quakePlotFilterStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy( stOld.stItem[i] );
      }
    }

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy the data structure
   * @param  stOld = The old data structure
   * @return stNew = The new data structure
   */

  public static quakePlotFilterStruct copy( quakePlotFilterStruct stOld )
  {
    quakePlotFilterStruct stNew = new quakePlotFilterStruct();

    if (stOld != null)
    {
      stNew.iBy        = stOld.iBy;                 // Filter By Indicator

      stNew.sKEY       = new String(stOld.sKEY);    // Primary Key

      stNew.depthStart = stOld.depthStart;          // Starting Depth
      stNew.depthEnd   = stOld.depthEnd;            // Ending Depth

      stNew.dataStart  = stOld.dataStart;           // Starting Data Value
      stNew.dataEnd    = stOld.dataEnd;             // Ending Data Value

      // Symbol and Color

      stNew.iEmpty     = stOld.iEmpty;              // Open or Filled Symbol
      stNew.iSymbol    = stOld.iSymbol;             // Symbol Identifier.
      stNew.iRed       = stOld.iRed;                // Red Color in RGB Setting
      stNew.iGreen     = stOld.iGreen;              // Green Color in RGB Setting
      stNew.iBlue      = stOld.iBlue;               // Blue Color in RGB Setting
      stNew.symbol     = new String(stOld.symbol);  // Label for the selected symbol.
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the data to the list
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static quakePlotFilterListStruct add(
	  quakePlotFilterStruct     stBlock,
	  quakePlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    quakePlotFilterListStruct stTemp = new quakePlotFilterListStruct();

    stTemp.iCount = 1;
    stTemp.stItem = new quakePlotFilterStruct[1];

    if (st != null)
    {
      stTemp.stItem = new quakePlotFilterStruct[st.iCount+1];

      for (i=0; i<st.iCount; i++)
      {
        stTemp.stItem[iCount] = copy( st.stItem[i] );
        iCount++;
      }

      st.delete();
      st = null;
    }

    stTemp.stItem[iCount] = copy( stBlock );
    iCount++;

    st        = new quakePlotFilterListStruct();
    st.stItem = new quakePlotFilterStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<iCount; i++)
    {
      st.stItem[i] = copy( stTemp.stItem[i] );
    }

    stTemp.delete();
    stTemp = null;

    return (st);
  }

  /** Method modify()
   * <p> This method will modify data within the list
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static quakePlotFilterListStruct modify(
	  quakePlotFilterStruct     stBlock,
	  quakePlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    quakePlotFilterListStruct stTemp = new quakePlotFilterListStruct();

    if (st != null)
    {
      stTemp.stItem = new quakePlotFilterStruct[st.iCount];

      for (i=0; i<st.iCount; i++)
      {
		if (!st.stItem[i].sKEY.equals(stBlock.sKEY))
          stTemp.stItem[iCount] = copy( st.stItem[i] );
        else
          stTemp.stItem[iCount] = copy( stBlock );

        iCount++;
      }

      st.delete();
      st = null;

      st        = new quakePlotFilterListStruct();
      st.stItem = new quakePlotFilterStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<iCount; i++)
      {
        st.stItem[i] = copy( stTemp.stItem[i] );
      }

      stTemp.delete();
      stTemp = null;
    }

    return (st);
  }

  /** Method remove()
   * <p> This method will remove the data from the list
   *     This method assumes that the data structure that
   *     is to be removed exists in the list.
   * @param  stBlock = the New Data Structure
   * @param  st      = the List Structure
   * @return st      = the Modified List structure
   */

  public static quakePlotFilterListStruct remove(
	  String                   sKEY,
	  quakePlotFilterListStruct st )
  {
	int i = 0;
	int iCount = 0;
    quakePlotFilterListStruct stTemp = null;

    if (st != null)
    {
	  if (st.iCount > 1)
	  {
	    stTemp        = new quakePlotFilterListStruct();
        stTemp.stItem = new quakePlotFilterStruct[st.iCount-1];

        for (i=0; i<st.iCount; i++)
        {
  		  if (!sKEY.equals(st.stItem[i].sKEY))
  		  {
            stTemp.stItem[iCount] = copy( st.stItem[i] );
            iCount++;
	      }
        }

        st.delete();
        st = null;

        st        = new quakePlotFilterListStruct();
        st.stItem = new quakePlotFilterStruct[iCount];
        st.iCount = iCount;

        for (i=0; i<iCount; i++)
        {
          st.stItem[i] = copy( stTemp.stItem[i] );
        }

        stTemp.delete();
        stTemp = null;
      }
      else
      {
	    st.delete();
	    st = null;
	  }
    }

    return (st);
  }

  /** Method print()
   *  <p> This method print the contents of List Structure
   *  @param  st      = the List Structure
   */

  public static void print(quakePlotFilterListStruct st)
  {
    if (st != null)
    {
	  for (int i=0; i<st.iCount; i++)
	  {
		System.out.println(
			st.stItem[i].iBy +
		    "KEY=" + st.stItem[i].sKEY          + " " +
		    "Depth: " + st.stItem[i].depthStart + "-" +
		                st.stItem[i].depthEnd   + " " +
		    "Data: "  + st.stItem[i].dataStart  + "-" +
		                st.stItem[i].dataEnd    + " " +
		    "Symbol=" +
		      plot.plotSymbolStruct.SHAPE_EMPTY[st.stItem[i].iSymbol] + " " +
		    "RGB="    + st.stItem[i].iRed   + " " +
		                st.stItem[i].iGreen + " " +
		                st.stItem[i].iBlue  + " " +
		    "Legend=" + st.stItem[i].symbol );
      }
    }
  }
}

/*
 *  @version 1.1 08/17/2015
 *  @author  John Victorine
 */
